package cn.com.ctop.kuaishou.modules.batch.mapper;

import cn.com.ctop.kuaishou.modules.batch.entity.DelKuaiShouVideoGet;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Set;

/**
 * @Description: 快手-获取视频接口
 * @Author: jeecg-boot
 * @Date: 2019-07-25
 * @Version: V1.0
 */
public interface DelKuaiShouVideoGetMapper extends BaseMapper<DelKuaiShouVideoGet> {

    void replaceBatch(@Param(value = "videos") List<DelKuaiShouVideoGet> videoGetList);

    void deleteByAccountId(@Param(value = "accountId") Long accountId);

    List<String> queryPhototIdsByPhotoName(@Param(value = "accountId") Long accountId, @Param(value = "photoName") String photoName);

    void replaceBatchVideoRelateCreatives(@Param("object") JSONObject object, @Param("creatives") JSONArray creatives);

    void replaceBatchVideoRelateAdvancedCreatives(@Param("object") JSONObject object, @Param("advancedCreativeIds") JSONArray advancedCreativeIds);

    List<Long> queryCreativeIds(@Param("accountId") Long accountId);

    Set<Long> queryAdvancedCreativeIds(@Param("accountId") Long accountId);

    void updateCreativesStatus(@Param("accountId") Long accountId, @Param("creativeIds") List<Long> creativeIds);

    void updateAdvancedCreativesStatus(@Param("accountId") Long accountId, @Param("unitIds") List<Long> unitIds);

    List<String> getPhotoIds(@Param("accountId") Long accountId, @Param("putStatus") String putStatus);

    List<String> getPhotoIds2(@Param("accountId") Long accountId, @Param("putStatus") String putStatus);

    void insertDelPhotoIdLog(@Param("accountId") Long accountId, @Param("photoIds") Set<String> photoIds, @Param("code") Integer code, @Param("message") String message);
}
